#cited-items h3,
#my-sources h3 {
    margin-bottom: 15px;
}

#cited-items {
    overflow-y: auto;
}

/* stylelint-disable selector-id-pattern */
#cite-source-table_filter {
    text-align: left;
}

#cite-source-table_filter input[type="text"] {
    width: 332px;
    height: 33px;
}
/* stylelint-enable selector-id-pattern */

#selected-cite-source-table .fw-data-table-title > span {
    width: 100px;
}

#cite-source-table .fw-data-table-title > span {
    width: 128px;
}

#configure-citation .datatable-container tbody tr {
    cursor: pointer;
}

#configure-citation .datatable-container tbody tr:nth-child(2n + 1) {
    background-color: var(--table-row-background-color);
}

.fw-cite-parts-table td {
    border: none;
    padding: 0;
}

.fw-cite-parts-table .cite-extra-fields {
    padding-bottom: 5px;
}

.cite-extra-fields > div {
    margin-top: 10px;
}

.cite-extra-fields label {
    display: inline-block;
    width: 40px;
    vertical-align: middle;
}

.cite-extra-fields input {
    width: 278px;
    border-radius: 4px;
}

.selected-source:first-child .order-up {
    display: none;
}

.selected-source:last-child .order-down {
    display: none;
}
