# Generated by Django 3.1.2 on 2020-12-09 15:10

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("document", "0005_auto_20201109_0255"),
    ]

    operations = [
        migrations.AlterField(
            model_name="document",
            name="diffs",
            field=models.JSONField(blank=True, default=list),
        ),
    ]
