.fw-contents .fw-flatpage h1 {
    font-size: 2em;
    margin: 0.67em 0;
    font-weight: 700;
}

.fw-contents .fw-flatpage h2 {
    font-size: 1.5em;
    margin: 0.75em 0;
    font-weight: 700;
}

.fw-contents .fw-flatpage h3 {
    font-size: 1.17em;
    margin: 0.83em 0;
    font-weight: 700;
}

.fw-contents .fw-flatpage h4 {
    margin: 1.12em 0;
    font-weight: 700;
}

.fw-contents .fw-flatpage h5 {
    font-size: 0.83em;
    margin: 1.5em 0;
    font-weight: 700;
}

.fw-contents .fw-flatpage h6 {
    font-size: 0.75em;
    margin: 1.67em 0;
    font-weight: 700;
}

.fw-contents .fw-flatpage ul {
    list-style-type: disc;
    list-style-position: inside;
}

.fw-contents .fw-flatpage ol {
    list-style-type: decimal;
    list-style-position: inside;
}
