# Generated by Django 5.1.7 on 2025-04-08 13:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user", "0011_delete_loginas_alter_user_options"),
    ]

    operations = [
        migrations.AddField(
            model_name="user",
            name="language",
            field=models.CharField(
                blank=True,
                choices=[
                    ("en", "English"),
                    ("bg", "Bulgarian"),
                    ("de", "German"),
                    ("fr", "French"),
                    ("it", "Italian"),
                    ("es", "Spanish"),
                    ("pt-br", "Portuguese (Brazil)"),
                ],
                default=None,
                help_text="Interface language preference",
                max_length=10,
                null=True,
            ),
        ),
    ]
