# Generated by Django 4.1.10 on 2023-09-28 09:57

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0010_alter_user_contacts"),
    ]

    operations = [
        migrations.DeleteModel(
            name="LoginAs",
        ),
        migrations.AlterModelOptions(
            name="user",
            options={
                "permissions": (("can_login_as", "Can login as another user"),)
            },
        ),
    ]
