# Generated by Django 3.2.4 on 2021-07-05 13:56

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("bibliography", "0002_move_json_data"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="entry",
            options={"verbose_name_plural": "Entries"},
        ),
    ]
